﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceProcess;
using System.Collections;
using System.Configuration.Install;
using System.Threading;
using System.Windows.Forms;

namespace ServiceHelper
{
    public static class ServiceControl
    {
        public static void Install(string serviceName, string serviceInstallPath)
        {
            IDictionary mySavedState = new Hashtable();

            try
            {
                ServiceController serviceController = new ServiceController(serviceName);

                if (Exist(serviceName))
                {
                    throw new Exception(string.Format("服务{0}已存在", serviceName));
                }

                ServiceHelper.bgWorker.ReportProgress(0);
                AssemblyInstaller assemblyInstaller = new AssemblyInstaller();
                mySavedState.Clear();
                assemblyInstaller.Path = serviceInstallPath;
                assemblyInstaller.UseNewContext = true;
                ServiceHelper.bgWorker.ReportProgress(0);
                assemblyInstaller.Install(mySavedState);
                ServiceHelper.bgWorker.ReportProgress(0);
                assemblyInstaller.Commit(mySavedState);
                ServiceHelper.bgWorker.ReportProgress(0);
                assemblyInstaller.Dispose();
                ServiceHelper.bgWorker.ReportProgress(0);
            }
            catch (Exception e)
            {
                throw new Exception("安装服务时出错：" + e.Message);
            }
        }

        public static void Uninstall(string serviceName, string serviceInstallPath)
        {
            try
            {
                if (!Exist(serviceName))
                {
                    throw new Exception(string.Format("服务{0}不存在", serviceName));
                }

                ServiceHelper.bgWorker.ReportProgress(0);
                AssemblyInstaller myAssemblyInstaller = new AssemblyInstaller();
                myAssemblyInstaller.UseNewContext = true;
                myAssemblyInstaller.Path = serviceInstallPath;
                ServiceHelper.bgWorker.ReportProgress(0);
                myAssemblyInstaller.Uninstall(null);
                ServiceHelper.bgWorker.ReportProgress(0);
                myAssemblyInstaller.Dispose();
                ServiceHelper.bgWorker.ReportProgress(0);
            }
            catch (Exception e)
            {
                throw new Exception("卸载服务时出错：" + e.Message);
            }
        }

        public static bool Exist(string serviceName)
        {
            ServiceController[] services = ServiceController.GetServices();
            foreach (ServiceController s in services)
            {
                if (s.ServiceName == serviceName)
                {
                    return true;
                }
            }
            return false;
        }

        public static void Start(string serviceName)
        {
            try
            {
                if (!Exist(serviceName))
                {
                    throw new Exception(string.Format("服务{0}不存在", serviceName));
                }
                ServiceHelper.bgWorker.ReportProgress(0);
                ServiceController service = new ServiceController(serviceName);
                if (service.Status != ServiceControllerStatus.Running && service.Status != ServiceControllerStatus.StartPending)
                {
                    ServiceHelper.bgWorker.ReportProgress(0);
                    service.Start();
                    for (int i = 0; i < 30; i++)
                    {
                        service.Refresh();
                        Thread.Sleep(1000);
                        if (service.Status == ServiceControllerStatus.Running)
                        {
                            service.Dispose();
                            break;
                        }
                        if (i == 29)
                        {
                            service.Dispose();
                            throw new Exception("服务" + serviceName + "启动失败！");
                        }
                    }
                }
                ServiceHelper.bgWorker.ReportProgress(0);
            }
            catch (Exception e)
            {
                throw new Exception("启动服务时出错：" + e.Message);
            }
            
        }

        public static void Stop(string serviceName)
        {
            try
            {
                if (!Exist(serviceName))
                {
                    throw new Exception(string.Format("服务{0}不存在", serviceName));
                }
                ServiceHelper.bgWorker.ReportProgress(0);
                ServiceController service = new ServiceController(serviceName);
                if (service.Status == ServiceControllerStatus.Running)
                {
                    ServiceHelper.bgWorker.ReportProgress(0);
                    service.Stop();
                    for (int i = 0; i < 30; i++)
                    {
                        service.Refresh();
                        System.Threading.Thread.Sleep(1000);
                        if (service.Status == ServiceControllerStatus.Stopped)
                        {
                            service.Dispose();
                            break;
                        }
                        if (i == 29)
                        {
                            service.Dispose();
                            throw new Exception("服务" + serviceName + "停止失败！");
                        }
                    }
                }
                ServiceHelper.bgWorker.ReportProgress(0);
            }
            catch (Exception e)
            {
                throw new Exception("停止服务失败：" + e.Message);
            }
        }

        public static void Restart(string serviceName)
        {
            try
            {
                Stop(serviceName);
                Start(serviceName);
            }
            catch (Exception e)
            {
                throw e;
            }
        }
    }
}
